import time
import warnings

import requests

from .exceptions import *

try:
    from json.decoder import JSONDecodeError
except ImportError:
    JSONDecodeError = ValueError


class Client(object):
    def __init__(self, base_url="https://darksearch.io", timeout=30, headers=None, proxies=None):
        self.base_url = base_url
        self.timeout = timeout
        self.headers = headers
        self.proxies = proxies

    def api_request(self, path, params=None, json=True):
        """
        :param path: Path to be requested
        :param base_url: Base URL to be combined with the path
        :param params: Base URL to be combined with the path
        :type path: str
        :type base_url: str
        :type params: dict
        :return: JSON Response
        :rtype: dict
        """
        try:
            response = requests.get(
                self.base_url + path, timeout=self.timeout, params=params, headers=self.headers, proxies=self.proxies)
            if response.status_code == 404:
                raise DarkSearchPageNotFound
            if response.status_code == 429:
                raise DarkSearchQuotaExceed
            if response.status_code == 504:
                raise DarkSearchServerError
            if json:
                return response.json()
            return response.content
        except requests.exceptions.RequestException:
            raise DarkSearchRequestException
        except JSONDecodeError:
            print(response.content)
            raise DarkSearchJSONDecodeException

    def api_search(self, query, page):
        """
        :param query: Query to be searched
        :param page: Page number of the search
        :type query: str
        :type page: int
        :return: search results
        :rtype: dict
        """
        if page < 1:
            page = 1
        return self.api_request("/api/search", params={"query": query, "page": page})

    def search(self, query, **kwargs):
        """
        :param query: Query to be searched
        :param page: Page number of the search
        :param pages: Pages to be returned
        :param wait: Time to wait between requests
        :type query: str
        :type page: int
        :type pages: int
        :type wait: int
        :return: search results
        :rtype: list or dict
        """
        page = kwargs.get("page")
        pages = kwargs.get("pages")
        wait = kwargs.get("wait")

        if pages and not page:
            results = list()
            try:
                for page_i in range(pages):
                    response = self.api_search(query, page_i + 1)
                    results.append(response)
                    if response.get("last_page") <= page_i + 1:
                        break
                    if wait:
                        time.sleep(wait)
            except DarkSearchQuotaExceed:
                warnings.warn(
                    "Seach Quota Exceeded, please keep searches to less than 30 per minute")
            return results
        if not page:
            page = 1
        return self.api_search(query, page)

    def crawling_status(self):
        """
        :return: crawling_status
        :rtype: int
        """
        return int(self.api_request("/api/crawling_status", json=False))



import argparse
from pprint import pprint

from darksearch import Client


def encode(source):
    try:
        if isinstance(source, unicode):
            return source.encode('utf8')
    except NameError:
        return str(source)


def print_response(response):
    print("Current Page: {}".format(response.get("current_page")))
    print("")
    print("From: {}".format(response.get("from")))
    print("To: {}".format(response.get("to")))
    print("Per Page: {}".format(response.get("per_page")))
    print("Last Page: {}".format(response.get("last_page")))
    print("")
    print("Results: ")
    for result in response.get("data"):
        print("Title: {}".format(encode(result.get("title"))))
        print("Description: {}".format(encode(result.get("description"))))
        print("Link: {}".format(result.get("link")))
        print("")


def main():
    parser = argparse.ArgumentParser(description="DarkSearch API Client")
    parser.add_argument("-q", "--query", help="search query")
    parser.add_argument("-p", "--page", type=int, help="page number")
    parser.add_argument("-n", "--pages", type=int, help="number of pages")
    parser.add_argument("-w", "--wait", type=int, help="wait between requests")
    parser.add_argument("-j", "--json", action="store_true",
                        help="prints as json")
    parser.add_argument("-v", "--verbose", action="store_true",
                        help="prints verbose json")
    args = parser.parse_args()

    if not args.query:
        parser.print_help()
        exit(1)

    client = Client(timeout=10)

    response = client.search(args.query, page=args.page,
                             pages=args.pages, wait=args.wait)

    if args.json:
        if args.verbose:
            print(response)
        else:
            pprint(response)
    else:
        if isinstance(response, list):
            for resp in response:
                print_response(resp)
        else:
            print_response(response)


if __name__ == "__main__":
    main()



class Dork(object):
    def __init__(self, *args, **kwargs):
        self.query = " ".join(args).strip()

        dorks = {
            "url": kwargs.get("url"),
            "title": kwargs.get("title"),
            "html_content": kwargs.get("html_content")
        }

        self.dorks = dorks

    def __str__(self):
        return "{0} {1}".format(str(self.query), self.parse_dorks(self.dorks)).strip()

    def combine_dorks(self, other):
        new_dorks = dict()
        for dorks in [self.dorks, other.dorks]:
            new_dorks.update((k, v)
                             for k, v in dorks.items() if v is not None)
        return new_dorks

    def AND(self, other):
        dorks = self.parse_dorks(self.combine_dorks(other))
        return self.parse_condition(self.query, "AND", other.query, dorks)

    def OR(self, other):
        dorks = self.parse_dorks(self.combine_dorks(other))
        return self.parse_condition(self.query, "OR", other.query, dorks)

    def NOT(self, other):
        dorks = self.parse_dorks(self.combine_dorks(other))
        return self.parse_condition(self.query, "NOT", other.query, dorks)

    @staticmethod
    def parse_condition(query_1, condition, query_2, dorks):
        return "\"{0}\" {1} \"{2}\" {3}".format(query_1, condition, query_2, dorks)

    @staticmethod
    def parse_dorks(dorks):
        dork_strings = list()
        for key, value in dorks.items():
            if value:
                dork_strings.append("{0}:\"{1}\"".format(key, str(value)))
        return " ".join(dork_strings)



from codecs import open
from os import path

from setuptools import setup

here = path.abspath(path.dirname(__file__))

with open(path.join(here, "README.rst"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name='DarkSearch',
    version='1.5.1',
    packages=['darksearch'],
    url='https://github.com/thehappydinoa/DarkSearch',
    license='MIT',
    author='thehappydinoa',
    author_email='thehappydinoa@gmail.com',
    description='Python API wrapper for darksearch.io',
    long_description=long_description,
    install_requires=[
        "requests",
        "pprint"
    ],
    entry_points={
        'console_scripts': ['darksearch=darksearch.cli:main'],
    },

    classifiers=[
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',

        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7'
    ]
)
